@@ -8,8 +8,12 @@ if logger.level == logging.NOTSET:
 
 from flask_assistant.core import (
     Assistant,
-    context_manager
+    context_manager,
+    intent,
+    request
 )
 
-from flask_assistant.response import ask, tell
+from flask_assistant.response import ask, tell, event, build_item
 from flask_assistant.manager import Context
+
+from api_ai.api import ApiAi
