@@ -1,16 +1,37 @@
 import inspect
 from functools import wraps, partial
 
+
 from flask import current_app, json, request as flask_request, _app_ctx_stack
 from werkzeug.local import LocalProxy
 
 from flask_assistant import logger
 from flask_assistant.response import _Response
 from flask_assistant.manager import ContextManager
+from api_ai.api import ApiAi
 
-request = LocalProxy(lambda: current_app.assist.request)
-context_in = LocalProxy(lambda: current_app.assist.context_in)
-context_manager = LocalProxy(lambda: current_app.assist.context_manager)
+
+def find_assistant():  # Taken from Flask-ask courtesy of @voutilad
+    """
+    Find our instance of Assistant, navigating Local's and possible blueprints.
+
+    Note: This only supports returning a reference to the first instance
+    of Assistant found.
+    """
+    if hasattr(current_app, 'assist'):
+        return getattr(current_app, 'assist')
+    else:
+        if hasattr(current_app, 'blueprints'):
+            blueprints = getattr(current_app, 'blueprints')
+            for blueprint_name in blueprints:
+                if hasattr(blueprints[blueprint_name], 'assist'):
+                    return getattr(blueprints[blueprint_name], 'assist')
+
+
+request = LocalProxy(lambda: find_assistant().request)
+intent = LocalProxy(lambda: find_assistant().intent)
+context_in = LocalProxy(lambda: find_assistant().context_in)
+context_manager = LocalProxy(lambda: find_assistant().context_manager)
 
 
 class Assistant(object):
@@ -31,9 +52,10 @@ class Assistant(object):
 
     """
 
-    def __init__(self, app=None, route='/'):
+    def __init__(self, app=None, blueprint=None, route=None):
 
         self.app = app
+        self.blueprint = blueprint
         self._route = route
         self._intent_action_funcs = {}
         self._intent_mappings = {}
@@ -44,8 +66,22 @@ class Assistant(object):
         self._context_funcs = {}
         self._func_contexts = {}
 
+        self.api = ApiAi()
+
+        if route is None and app is not None:
+
+            logger.warn("""WARNING:
+                No route was provided for the Assistant object, but a flask `app` object given
+                The Assistant will be mapped to the app's '/' endpoint.
+                If this is a problem please initialize the Assitant with the 'route' parameter
+                """)
+
+            self._route = '/'
+
         if app is not None:
             self.init_app(app)
+        elif blueprint is not None:
+            self.init_blueprint(blueprint)
 
     def init_app(self, app):
 
@@ -55,6 +91,29 @@ class Assistant(object):
         app.assist = self
         app.add_url_rule(self._route, view_func=self._flask_assitant_view_func, methods=['POST'])
 
+    # Taken from Flask-ask courtesy of @voutilad
+    def init_blueprint(self, blueprint, path='templates.yaml'):
+        """Initialize a Flask Blueprint, similar to init_app, but without the access
+        to the application config.
+
+        Keyword Arguments:
+            blueprint {Flask Blueprint} -- Flask Blueprint instance to initialize (Default: {None})
+            path {str} -- path to templates yaml file, relative to Blueprint (Default: {'templates.yaml'})
+        """
+        if self._route is not None:
+            raise TypeError("route cannot be set when using blueprints!")
+
+        # we need to tuck our reference to this Ask instance into the blueprint object and find it later!
+        blueprint.assist = self
+
+        # BlueprintSetupState.add_url_rule gets called underneath the covers and
+        # concats the rule string, so we should set to an empty string to allow
+        # Blueprint('blueprint_api', __name__, url_prefix="/assist") to result in
+        # exposing the rule at "/assist" and not "/assist/".
+        blueprint.add_url_rule("", view_func=self._flask_assitant_view_func, methods=['POST'])
+        # blueprint.jinja_loader = ChoiceLoader([YamlLoader(blueprint, path)])
+
+
     @property
     def request(self):
         """Local Proxy refering to the request JSON recieved from API.AI"""
@@ -121,20 +180,20 @@ class Assistant(object):
             return wrapper
         return decorator
 
-    def action(self, intent, mapping={}, convert={}, default={}, with_context=[], *args, **kw):
-        """Decorates an intent's Action view function.
+    def action(self, intent_name, mapping={}, convert={}, default={}, with_context=[], *args, **kw):
+        """Decorates an intent_name's Action view function.
 
             The wrapped function is called when a request with the
             given intent_name is recieved along with all required parameters.
         """
         def decorator(f):
-            action_funcs = self._intent_action_funcs.get(intent, [])
+            action_funcs = self._intent_action_funcs.get(intent_name, [])
             action_funcs.append(f)
-            self._intent_action_funcs[intent] = action_funcs
+            self._intent_action_funcs[intent_name] = action_funcs
 
-            self._intent_mappings[intent] = mapping
-            self._intent_converts[intent] = convert
-            self._intent_defaults[intent] = default
+            self._intent_mappings[intent_name] = mapping
+            self._intent_converts[intent_name] = convert
+            self._intent_defaults[intent_name] = default
 
             @wraps(f)
             def wrapper(*args, **kw):
@@ -142,7 +201,7 @@ class Assistant(object):
             return f
         return decorator
 
-    def prompt_for(self, next_param, intent):
+    def prompt_for(self, next_param, intent_name):
         """Decorates a function to prompt for an action's required parameter.
 
         The wrapped function is called if next_param was not recieved with the given intent's
@@ -153,12 +212,12 @@ class Assistant(object):
             intent_name {str} -- name of the intent the dependent action belongs to
         """
         def decorator(f):
-            prompts = self._intent_prompts.get(intent)
+            prompts = self._intent_prompts.get(intent_name)
             if prompts:
                 prompts[next_param] = f
             else:
-                self._intent_prompts[intent] = {}
-                self._intent_prompts[intent][next_param] = f
+                self._intent_prompts[intent_name] = {}
+                self._intent_prompts[intent_name][next_param] = f
 
             @wraps(f)
             def wrapper(*args, **kw):
@@ -180,8 +239,12 @@ class Assistant(object):
     def _dump_view_info(self, view_func=lambda: None):
         _infodump('Result: Matched {} intent to {} func'.format(self.intent, view_func.__name__))
 
-    def _flask_assitant_view_func(self, *args, **kwargs):
-        self.request = self._api_request(verify=False)
+    def _flask_assitant_view_func(self, nlp_result=None, *args, **kwargs):
+        if nlp_result: # pass API query result directly
+            self.request = nlp_result
+        else: # called as webhook
+            self.request = self._api_request(verify=False)
+
         _dbgdump(self.request['result'])
 
         self.intent = self.request['result']['metadata']['intentName']
@@ -297,9 +360,9 @@ class Assistant(object):
             # param name cant have '.',
             # so when registered, the sys. is stripped,
             # and must be stripped when looking up in request
-            mapped_param_name = entity_mapping.replace('sys.', '') 
+            mapped_param_name = entity_mapping.replace('sys.', '')
             value = params.get(mapped_param_name)  # params declared in GUI present in request
-            
+
             if not value:  # params not declared, so must look in contexts
                 value = self._map_arg_from_context(arg_name)
             arg_values.append(value)
